/* See somegrids.h */
/* Last edited on 2005-06-05 19:20:54 by stolfi */

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>

#include <dg_grid.h>
#include <dg_tree.h>
#include <bz_patch.h>
#include <dg_makegrid.h>
#include <affirm.h>
#include <bool.h>

#include <somegrids.h>
#include <figtools.h>

/* INTERNAL PROTOTYPES */

double local_tol(double x, double y);
  /* Maximum box radius allowed near {p}. */ 

double inside(double x, double y, double rad);
  /* TRUE if a box of radius {rad} centered at {p} intersects the 
    desired domain. */ 

double dsoft(double x1, double y1, double x2, double y2, double eps);
  /* Distance from {(x1,y1)} to {(x2,y2)} on the canonical
    torus, smoothly cooked to be at least {eps}. */

/* ADAPTIVE TREE GENERATION */

#define tol_eps (0.001)

#define x_well_1 (0.80)
#define y_well_1 (0.60)

#define x_well_2 (0.25)
#define y_well_2 (0.15)

#define x_fault (0.30)
#define y_fault (0.30)
#define r_fault (0.10)

double local_tol(double x, double y)
  { /* double d1 = dsoft(x, y, x_well_1, y_well_1, tol_eps); */
    double d1 = 100000.00;
    /* double d2 = dsoft(x, y, x_well_2, y_well_2, tol_eps); */
    double d2 = 100000.00;
    double df = dsoft(x, y, x_fault, y_fault, 0.000) - r_fault;
    double d3 = sqrt(df*df + tol_eps*tol_eps);
    /* double d3 = 100000.00; */
    double maxr = 0.25 * 3.0/(1.0/d1 + 1.0/d2 + 1.0/d3);
    return maxr;
  }

double inside(double x, double y, double rad)
  { double dctr = hypot(x - 0.5, y - 0.5);
    return dctr - rad < 0.4; 
  }

#define SQRT_HALF 0.7071067811865475

double dsoft(double x1, double y1, double x2, double y2, double eps)
  { double dx = fabs(x2 - x1);
    double dy = fabs(y2 - y1);
    while (dx > 1.0) { dx -= 1.0; }
    while (dy > SQRT_HALF) { dy -= SQRT_HALF; }
    if (dx > 0.5) { dx = 1.0 - dx; }
    if (dy > 0.5*SQRT_HALF) { dy = SQRT_HALF - dy; }
    return sqrt(dx*dx + dy*dy + eps*eps);
  }

dg_tree_node_t *somegrid_adaptive(dg_dim_t d, dg_rank_t maxDp, bz_patch_t *b)
  {
    interval_t bbox[MAX_RDIM];
    double ctr[MAX_RDIM];
    int n = (b == NULL ? 2 : b->n);

    auto bool_t omit(dg_cell_index_t k, dg_rank_t r, bz_patch_t *b, dg_tree_node_t *p);
      /* Tells {dg_make_grid} when to discard a cell. */ 

    auto bool_t split(dg_cell_index_t k, dg_rank_t r, bz_patch_t *b, dg_tree_node_t *p);
      /* Tells {dg_make_grid} when to split a cell. Basically, when
        the variation in the coordinates and function values within
        the cell is larger than some tolerance, which depends on the
        cell's center. */ 

    bool_t omit(dg_cell_index_t k, dg_rank_t r, bz_patch_t *b, dg_tree_node_t *p)
      { 
        /* fprintf(stderr, "  omit k = %lld\n", k); */
        return FALSE;
      }

    bool_t split(dg_cell_index_t k, dg_rank_t r, bz_patch_t *b, dg_tree_node_t *p)
      { int i; double r2 = 0.0;
        double rad, radMax;
        if (r >= maxDp) return FALSE;
        if (b != NULL)
          { bz_patch_compute_bbox(b, bbox); }
        else
          { dg_cell_box_canonical(d, k, bbox); } 
        fprintf(stderr, "  split k = %lld = ", k);
        print_2d_cell(stderr, bbox);
        for (i = 0; i < n; i++) 
          { interval_t bi = bbox[i];
            double ri = (HI(bi) - LO(bi))/2; /* Half-width of interval. */
            ctr[i] = (LO(bi) + HI(bi))/2;
            r2 += ri*ri;
          }
        rad = sqrt(r2);
        radMax = local_tol(ctr[0], ctr[1]);
        fprintf(stderr, "  rad = %7.4f radMax = %7.4f\n", rad, radMax);
        return rad >= radMax;
      }

    return dg_make_grid(d, dg_ROOT_CELL, 0, b, NULL, &omit, &split);
  }

dg_tree_node_t *somegrid_varied(dg_dim_t d, dg_rank_t splitDp, dg_rank_t maxDp, bz_patch_t *b)
  {
    auto bool_t omit(dg_cell_index_t k, dg_rank_t r, bz_patch_t *b, dg_tree_node_t *p);
      /* Tells {dg_make_grid} when to discard a cell. */ 

    auto bool_t split(dg_cell_index_t k, dg_rank_t r, bz_patch_t *b, dg_tree_node_t *p);
      /* Tells {dg_make_grid} when to split a cell. Basically, 
        a random function of the parent so that about 3/4 of the cells
        split at each level. */ 

    bool_t omit(dg_cell_index_t k, dg_rank_t r, bz_patch_t *b, dg_tree_node_t *p)
      { 
        /* fprintf(stderr, "  omit k = %lld\n", k); */
        return FALSE;
      }

    dg_cell_index_t firstCell[DG_MAX_RANK];
    dg_rank_t r;
    for (r = 0; r <= DG_MAX_RANK; r++) { firstCell[r] = DG_NO_CELL; }
    
    bool_t split(dg_cell_index_t k, dg_rank_t r, bz_patch_t *b, dg_tree_node_t *p)
      { /* Always split cells up to {splitDp}: */
        if (r < splitDp) { return TRUE; }
        /* Stop at depth {maxDp}: */
        if (r >= maxDp) { return FALSE; }
        
        if (firstCell[r] == DG_NO_CELL)
          { /* First cell in this level, split it: */
            firstCell[r] = k; return TRUE;
          }
        else if (firstCell[r]+1 == k)
          { /* Second cell, split it only in level {splitDp+1}: */
            return (r == splitDp + 1);
          }
        /* Split randomly 6/7 of the cells. */
        dg_cell_index_t parent = (k >> 1);
        double burp = ((double)parent) * ((sqrt(5) + 1.0)/2.0);
        burp -= floor(burp);
        dg_cell_index_t scramble = (int)floor(8*burp);
        return ((k & 7L) != (scramble & 7L));
      }

    return dg_make_grid(d, dg_ROOT_CELL, 0, b, NULL, &omit, &split);
  }

